/**
 * Copyright (c) Microsoft Corporation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
import { Dispatcher } from './dispatcher';
import { ElementHandleDispatcher } from './elementHandlerDispatcher';
import { parseSerializedValue, serializeValue } from '../../protocol/serializers';
export class JSHandleDispatcher extends Dispatcher {
    _type_JSHandle = true;
    static fromJSHandle(scope, handle) {
        return scope.connection.existingDispatcher(handle) || new JSHandleDispatcher(scope, handle);
    }
    constructor(scope, jsHandle) {
        // Do not call this directly, use createHandle() instead.
        super(scope, jsHandle, jsHandle.asElement() ? 'ElementHandle' : 'JSHandle', {
            preview: jsHandle.toString(),
        });
        jsHandle._setPreviewCallback(preview => this._dispatchEvent('previewUpdated', { preview }));
    }
    async evaluateExpression(params, progress) {
        const jsHandle = await this._object.evaluateExpression(progress, params.expression, { isFunction: params.isFunction }, parseArgument(params.arg));
        return { value: serializeResult(jsHandle) };
    }
    async evaluateExpressionHandle(params, progress) {
        const jsHandle = await this._object.evaluateExpressionHandle(progress, params.expression, { isFunction: params.isFunction }, parseArgument(params.arg));
        // If "jsHandle" is an ElementHandle, it belongs to the same frame as "this".
        return { handle: ElementHandleDispatcher.fromJSOrElementHandle(this.parentScope(), jsHandle) };
    }
    async getProperty(params, progress) {
        const jsHandle = await this._object.getProperty(progress, params.name);
        // If "jsHandle" is an ElementHandle, it belongs to the same frame as "this".
        return { handle: ElementHandleDispatcher.fromJSOrElementHandle(this.parentScope(), jsHandle) };
    }
    async getPropertyList(params, progress) {
        const map = await this._object.getProperties(progress);
        const properties = [];
        for (const [name, value] of map) {
            // If "jsHandle" is an ElementHandle, it belongs to the same frame as "this".
            properties.push({ name, value: ElementHandleDispatcher.fromJSOrElementHandle(this.parentScope(), value) });
        }
        return { properties };
    }
    async jsonValue(params, progress) {
        return { value: serializeResult(await this._object.jsonValue(progress)) };
    }
    async dispose(_, progress) {
        progress.metadata.potentiallyClosesScope = true;
        this._object.dispose();
        this._dispose();
    }
}
// Generic channel parser converts guids to JSHandleDispatchers,
// and this function takes care of converting them into underlying JSHandles.
export function parseArgument(arg) {
    return parseSerializedValue(arg.value, arg.handles.map(a => a._object));
}
export function parseValue(v) {
    return parseSerializedValue(v, []);
}
export function serializeResult(arg) {
    return serializeValue(arg, value => ({ fallThrough: value }));
}
//# sourceMappingURL=jsHandleDispatcher.js.map